@@ -36,12 +36,6 @@ class Deb822Dict(UserDict.DictMixin):
     # Subclassing UserDict.DictMixin because we're overriding so much dict
     # functionality that subclassing dict requires overriding many more than
     # the four methods that DictMixin requires.
-    #
-    # HACK: I'm also subclassing dict, but all the methods we care about will
-    # already be provided by DictMixin.  This way, Deb822Dict is a "real"
-    # subclass of dict, i.e.
-    #     isinstance(Deb822Dict(), dict) == True
-    # but we get out of implementing lots of methods.
     """A dictionary-like object suitable for storing RFC822-like data.
 
     Deb822Dict behaves like a normal dict, except:
@@ -132,7 +126,8 @@ class Deb822Dict(UserDict.DictMixin):
         return True
 
     def copy(self):
-        copy = Deb822Dict(self)
+        # Use self.__class__ so this works as expected for subclasses
+        copy = self.__class__(self)
         return copy
 
     # TODO implement __str__() and make dump() use that?
@@ -180,7 +175,7 @@ class Deb822(Deb822Dict):
             with apt_pkg.) Set this parameter to False to disable using apt_pkg. 
         """
 
-        # TODO Think about still using apt_pkg evein if shared_storage is False,
+        # TODO Think about still using apt_pkg even if shared_storage is False,
         # by somehow instructing the constructor to make copy of the data. (If
         # this is still faster.)
 
@@ -265,9 +260,10 @@ class Deb822(Deb822Dict):
         else:
             return_string = False
         for key, value in self.iteritems():
-            if value[0] == '\n':
+            if not value or value[0] == '\n':
                 # Avoid trailing whitespace after "Field:" if it's on its own
-                # line
+                # line or the value is empty
+                # XXX Uh, really print value if value == '\n'?
                 fd.write('%s:%s\n' % (key, value))
             else:
                 fd.write('%s: %s\n' % (key, value))
@@ -435,18 +431,23 @@ class _multivalued(Deb822):
         else:
             return_string = False
         for key in self.keys():
-            if key not in self._multivalued_fields:
-                # normal behavior
-                fd.write(key + ": " + self[key] + "\n")
+            keyl = key.lower()
+            if keyl not in self._multivalued_fields:
+                value = self[key]
+                if not value or value[0] == '\n':
+                    # XXX Uh, really print value if value == '\n'?
+                    fd.write('%s:%s\n' % (key, value))
+                else:
+                    fd.write('%s: %s\n' % (key, value))
             else:
                 fd.write(key + ":")
                 if isinstance(self[key], dict): # single-line
                     array = [ self[key] ]
                 else: # multi-line
-                    fd.write(" \n")
+                    fd.write("\n")
                     array = self[key]
 
-                order = self._multivalued_fields[key]
+                order = self._multivalued_fields[keyl]
                 for item in array:
                     fd.write(" " + " ".join([item[x] for x in order]))
                     fd.write("\n")
